﻿using System;

namespace DataBase
{
    /// <summary>
    /// DataBaseManager 는 DB에 대한 처리를 전체적으로 관리하는 매니져이다.
    /// 싱글톤으로 되어 있기 때문에 1개만 생성이 가능하며 어떠한 코드에서도 쉽게 접근 할 수 있다.
    /// </summary>
    sealed class DataBaseManager
    {
        /// <summary>
        /// 싱글톤 선언 코드
        /// </summary>
        private static volatile DataBaseManager instance;
        private static object syncRoot = new Object();

        private DataBaseManager() { }

        public static DataBaseManager Instance
        {
            get
            {
                if (instance == null)
                {
                    lock (syncRoot)
                    {
                        if (instance == null)
                            instance = new DataBaseManager();
                    }
                }

                return instance;
            }
        }
        // 싱글톤 끝

        // 추후 확장성을 대비하여 여러 가지 DB를 지원하도록 한다.
        public enum DBType
        {
            MSSQL = 0,
            Access = 1,
            Oracle = 2,
        }

        public IDataManager DBMain = null;
        public Account DBAccount = null;
 
        /// <summary>
        /// DB에 접속을 한다. 접속이 성공하면 인스턴스들을 생성해준다.
        /// </summary>
        public bool DBConnect(DBType dbType)
        {
            if(DBMain == null)
            {
                switch(dbType)
                {
                    case DBType.MSSQL:
                        DBMain = new SqlDataManager(); // SQL DB 생성
                        break;
                }
            }

            if (DBMain != null)
            {
                // 각 sql db에 맞는 정보를 넣어야한다.
                bool result = DBMain.DBInitConnect("dingpong.iptime.org", "OWNER2", "sa", "password", "dingpong");
                if(result == true)
                {
                    if(DBAccount == null)
                    {
                        DBAccount = new Account(); // 계정에 대한 DB 처리
                    }
                }
                return result;
            }
            return true;
      }
    }
}