﻿using System;
using System.Collections;
using System.Data;

using GameLogic;

namespace DataBase
{
    /// <summary>
    /// 계정에 대한 DB 요청을 처리한다.
    /// </summary>
    public class Account
    {
        // 계정에 대한 데이터를 DB에 추가해준다.
        public bool CreateAccount(string id, string emailAddress)
        {
            string activeCode = LogicManager.Instance.Util.GetRandomPassword(14, 1);
            string password = LogicManager.Instance.Util.GetRandomPassword(14, 10);

            ArrayList fieldData = new ArrayList();
            fieldData.Add("id");
            fieldData.Add("password");
            fieldData.Add("email");
            fieldData.Add("activeCode");
            fieldData.Add("isActive");
            fieldData.Add("loginCount");
            fieldData.Add("lastLoginDate");
            ArrayList insertData = new ArrayList();
            insertData.Add(id);
            insertData.Add(password);
            insertData.Add(emailAddress);
            insertData.Add(activeCode);
            insertData.Add(0);
            insertData.Add(0);
            insertData.Add(DateTime.Now.GetDateTimeFormats()[86]);

            if (DataBaseManager.Instance.DBMain.InsertRecodeInTable("Account", fieldData, insertData) != 0)
            {
                LogicManager.Instance.emailSend.SendCreateAccount(emailAddress, activeCode, password);
                // 계정 추가에 성공하면 true를 리턴한다.
                return true;
            }
            return false;
        }
        /// <summary>
        /// 계정이 현재 존재하는 계정인지 체크하여 존재하면 true를 리턴한다.
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public bool IsAccountExistID(string id)
        {
            DataTableCollection collection = DataBaseManager.Instance.DBMain.GetTableCollection("Account", new ArrayList(), "id = '" + id + "'", null);
            if (collection["Account"].Rows.Count == 0)
                return false;

            return true;
        }
        /// <summary>
        /// 계정 이메일이 등록 되어 있는 상태인지 확인한다.
        /// </summary>
        /// <param name="emailAddress"></param>
        /// <returns></returns>
        public bool IsAccountExistEmail(string emailAddress)
        {
            DataTableCollection collection = DataBaseManager.Instance.DBMain.GetTableCollection("Account", new ArrayList(), "email = '" + emailAddress, "'");
            if (collection["Account"].Rows.Count == 0)
                return false;

            return true;
        }
        /// <summary>
        /// 계정이 현재 Active 상태인지 확인한다.
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public bool IsAuccountActive(string id)
        {
            DataTableCollection collection = DataBaseManager.Instance.DBMain.GetTableCollection("Account", new ArrayList(), "id = '" + id + "'", null);
            if (collection["Account"].Rows.Count == 0)
            {
                return false;
            }
            else
            {
                if ((bool)collection["Account"].Rows[0][4] == false)  // account 의 isActive 값을 얻어서 비교한다.
                {
                    return false;
                }
                else
                {
                    return true;
                }
            }
        }
        /// <summary>
        /// 계정을 Active 상태로 바꾸는 것을 요청한다.
        /// </summary>
        /// <param name="id"></param>
        /// <param name="activeCode"></param>
        /// <returns></returns>
        public bool RequestAccountActive(string id, string activeCode)
        {
            DataTableCollection collection = DataBaseManager.Instance.DBMain.GetTableCollection("Account", new ArrayList(), "id = '" + id + "'", null);
            if (collection["Account"].Rows.Count == 0)
            {
                return false;
            }
            else
            {
                if (collection["Account"].Rows[0][3].ToString() == activeCode) // account 의 activeCode 값을 얻어서 비교한다.
                {
                    ArrayList fieldData = new ArrayList();
                    fieldData.Add("isActive");
                    ArrayList updateData = new ArrayList();
                    updateData.Add("1");
                    DataBaseManager.Instance.DBMain.UpdateRecodeInTable("Account", fieldData, updateData, "id = '" + id + "'");
                    return true;
                }
                else
                {
                    return false;
                }
            }
        }
        /// <summary>
        /// 현재 id 가 active 상태인지 확인한다.
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public bool IsIDActive(string id)
        {
            DataTableCollection collection = DataBaseManager.Instance.DBMain.GetTableCollection("Account", new ArrayList(), "id = '" + id + "'", null);
            if (collection["Account"].Rows.Count == 0)
            {
                return false;
            }
            else
            {
                if ((bool)collection["Account"].Rows[0][4] == false) // account 의 isactive 값을 비교한다.
                {
                    return false;
                }
                else
                {
                    return true;
                }
            }
        }
        /// <summary>
        /// id와 패스워드 정보가 맞는지 비교한다.
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public bool IsIDPasswordCompare(string id, string password)
        {
            DataTableCollection collection = DataBaseManager.Instance.DBMain.GetTableCollection("Account", new ArrayList(), "id = '" + id + "'", null);
            if (collection["Account"].Rows.Count == 0)
            {
                return false;
            }
            else
            {
                if (collection["Account"].Rows[0][1].ToString() == password) // account 의 id와 password를 비교한다.
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
        }

        /// <summary>
        /// 로그인이 성공적으로 되어 로그인 데이터를 요청한다.
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public int UpdateLogin(string id)
        {
            DataTableCollection collection = DataBaseManager.Instance.DBMain.GetTableCollection("Account", new ArrayList(), "id = '" + id + "'", null);
            if (collection["Account"].Rows.Count == 0)
            {
                return 0;
            }
            else
            {
                ArrayList fieldData = new ArrayList();
                fieldData.Add("loginCount");
                fieldData.Add("lastLoginDate");
                ArrayList updateData = new ArrayList();
                updateData.Add((int)collection["Account"].Rows[0][5] + 1);
                updateData.Add(DateTime.Now.GetDateTimeFormats()[86]);

                return DataBaseManager.Instance.DBMain.UpdateRecodeInTable("Account", fieldData, updateData, "id = '" + id + "'");
            }
        }
        // 요청이 들어온 계정을 DB에서 삭제해준다.
        public bool DeleteAccount()
        {
            return true;
        }
    }
}
