﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Text;
using System.Net;
using System.Threading;
using System.Xml;
using System.Xml.Serialization;
using System.IO;

using Network;
using LogicData;
using DataBase;

namespace GameLogic
{
    public interface INetworkMsg
    {
        void GetMsgIP(String d, string IPFrom);
    }

    public interface INetworkServerMsg : INetworkMsg
    {
        int Port { get; }
    }

    public class NetworkMsg : INetworkServerMsg
    {
        public void GetMsgIP(String d, string IPFrom)
        {
            Console.WriteLine("ip : " + IPFrom);
            Console.WriteLine(d);
            
            dStringIP o = null;

            if (d.Contains("LoginData"))
            {
                o = new dStringIP(ProcessLogin);
            }
            else if (d.Contains("ChatData"))
            {
                o = new dStringIP(ProcessChat);
            }
            else if (d.Contains("RegisterData"))
            {
                o = new dStringIP(ProcessRegister);
            }
            else if (d.Contains("ActiveCodeData"))
            {
                o = new dStringIP(ProcessActiveCode);
            }
            if (o != null) o(d, IPFrom);
        }

        public void WriteStartInfo(String d)
        {
            Console.WriteLine("{0} startInfo", d);
        }

        /// <summary>
        /// 로그인 메세지를 받았다. DB 에서 로그인 정보와 비교해서 로그인이 되면 클라이언트에게 알려준다.
        /// </summary>
        /// <param name="d"></param>
        /// <param name="IPFrom"></param>
        public void ProcessLogin(String d, string IPFrom)
        {
            LoginDataManager login = new LoginDataManager(d);

            if (DataBaseManager.Instance.DBAccount.IsAccountExistID(login.loginData.ID) == false)
            {
                SendChatMsg("존재하지 않는 ID 에 대한 로그인입니다.", "ERROR", ChatType.SystemChat, IPFrom);
                return;
            }
            else
            {
                if (DataBaseManager.Instance.DBAccount.IsIDActive(login.loginData.ID) == false)
                {
                    SendChatMsg("활성화 되어 있지 않은 ID 에 대한 로그인입니다.", "ERROR", ChatType.SystemChat, IPFrom);
                    return;
                }
                else
                {
                    if (DataBaseManager.Instance.DBAccount.IsIDPasswordCompare(login.loginData.ID, login.loginData.Password) == false)
                    {
                        SendChatMsg("로그인 ID와 PASSWORD가 맞지 않습니다.", "ERROR", ChatType.SystemChat, IPFrom);
                        return;
                    }
                    else
                    {
                        if (DataBaseManager.Instance.DBAccount.UpdateLogin(login.loginData.ID) != 0)
                        {
                            LogicManager.Instance.Util.FindClient(IPFrom).playerData.ID = login.loginData.ID;
                            SendChatMsg("성공적으로 로그인이 되었습니다.", "SYSTEM", ChatType.SystemChat, IPFrom);
                            SendLoginSuccessMsg(login.loginData.ID, 0, IPFrom);
                            Console.WriteLine("login, id = {0} pw = {1}", login.loginData.ID, login.loginData.Password);
                            return;
                        }
                        else
                        {
                            SendChatMsg("로그인에 실패하였습니다. 관리자에게 문의해주세요.", "ERROR", ChatType.SystemChat, IPFrom);
                            return;
                        }
                    }
                }
            }
        }
        /// <summary>
        /// 로그인 되었을 때 메세지를 받아서 보낸 사람 외에 다른 모든 사람에게 다시 전송해준다.
        /// </summary>
        /// <param name="d"></param>
        /// <param name="IPFrom"></param>
        public void ProcessChat(String d, string IPFrom)
        {
            // 전송 받은 내용 그대로 다시 클라이언트들에게 보내준다.
            LogicManager.Instance.server.SendToAll(d);
//            Console.WriteLine("chat, chatMsg = {0} nickName = {1}", chat.chatData.ChatMsg, chat.chatData.NickName);
        }
        /// <summary>
        /// 회원 가입 관련 요청을 받았다.
        /// </summary>
        /// <param name="d"></param>
        /// <param name="IPFrom"></param>
        public void ProcessRegister(String d, string IPFrom)
        {
            RegisterDataManager register = new RegisterDataManager(d);
            if (DataBaseManager.Instance.DBAccount.IsAccountExistID(register.RegisterData.ID) == true)
            {
                SendChatMsg("이미 존재하는 ID 입니다.", "ERROR", ChatType.SystemChat, IPFrom);
                return;
            }
            else if (DataBaseManager.Instance.DBAccount.IsAccountExistEmail(register.RegisterData.EMailAddress) == true)
            {
                SendChatMsg("이미 존재하는 E-Mail 입니다.", "ERROR", ChatType.SystemChat, IPFrom);
                return;
            }
            else
            {
                // 계정 생성 요청
                if (DataBaseManager.Instance.DBAccount.CreateAccount(register.RegisterData.ID, register.RegisterData.EMailAddress) == true)
                {
                    SendChatMsg("계정 생성에 성공하였습니다. E-Mail 을 확인해주세요.", "SYSTEM", ChatType.SystemChat, IPFrom);
                    return;
                }
                else
                {
                    SendChatMsg("계정 생성에 실패하였습니다. 관리자에게 문의해주세요.", "ERROR", ChatType.SystemChat, IPFrom);
                    return;
                }
            }
        }

        /// <summary>
        /// 계정 활성화 코드를 입력받았다.
        /// </summary>
        /// <param name="d"></param>
        /// <param name="IPFrom"></param>
        public void ProcessActiveCode(String d, string IPFrom)
        {
            ActiveCodeDataManager activeCodeManager = new ActiveCodeDataManager(d);
            if (DataBaseManager.Instance.DBAccount.IsAccountExistID(activeCodeManager.ActiveCodeData.ID) == false)
            {
                SendChatMsg("존재하지 않는 ID 에 대한 요청입니다.", "ERROR", ChatType.SystemChat, IPFrom);
                return;
            }
            else
            {
                if (DataBaseManager.Instance.DBAccount.IsAuccountActive(activeCodeManager.ActiveCodeData.ID) == true)
                {
                    SendChatMsg("이미 Active 된 계정입니다.", "ERROR", ChatType.SystemChat, IPFrom);
                    return;
                }
                else
                {
                    if (DataBaseManager.Instance.DBAccount.RequestAccountActive(activeCodeManager.ActiveCodeData.ID, activeCodeManager.ActiveCodeData.ActiveCode) == false)
                    {
                        SendChatMsg("잘못된 Code 입니다. 다시 한번 확인해주세요.", "ERROR", ChatType.SystemChat, IPFrom);
                        return;
                    }
                    else
                    {
                        SendChatMsg("계정이 성공적으로 Active 되었습니다.", "SYSTEM", ChatType.SystemChat, IPFrom);
                        return;
                    }
                }
            }
         }

        
        public void WriteMsg(String d)
        {
            Console.WriteLine(d);
        }

        /// <summary>
        /// 클라이언트로 쳇 메세지를 보낸다.
        /// </summary>
        /// <param name="chatMsg"></param>
        /// <param name="nickName"></param>
        /// <param name="type"></param>
        /// <param name="IPFrom"></param>
        public void SendChatMsg(string chatMsg, string nickName, ChatType type, string IPFrom)
        {
            ChatDataManager chatDataManager = new ChatDataManager(chatMsg, nickName, type);
            LogicManager.Instance.server.SendToOne(chatDataManager.GetXML(), LogicManager.Instance.Util.FindClient(IPFrom));
        }
        public void SendLoginSuccessMsg(string id, int loginCount, string IPFrom)
        {
            LoginSuccessDataManager loginSuccessDataManager = new LoginSuccessDataManager(id, loginCount);
            LogicManager.Instance.server.SendToOne(loginSuccessDataManager.GetXML(), LogicManager.Instance.Util.FindClient(IPFrom));
        }

        // IChatServerUI 구현
        public int Port
        {
            get
            {
                return 4530;
            }
        }
    }
}
